/**
* \file uart.h
* @brief Konfigurationsdatei zur seriellen Schnittstelle
* \author Lars Neubert
* \date Feb 2010
*/

/**
* definiert Mikrocontroller-Taktfrequenz falls diese nicht im Makefile definiert wurde
*/
#ifndef F_CPU
	#define F_CPU 8000000UL 	       
#endif

#include <avr/interrupt.h>
#include <avr/io.h>
#include <string.h>
#include <stdio.h>
#include <util/delay.h>

// nderung 04.03.2010, nicht mehr ntig, da in ISR keine Variablen aus sms.h genutzt werden
//#include "sms.h"


/**
* definiert Baudrate fr serielle Schnittstelle
*/
#define UART_BAUD 19200UL


/**
* Berechnet anhand der Taktfrequnz des Mikrocontrollers und der gewhlten Baudrate die Einstellung fr das Baudratenregister.
*/
#define UART_UBRR_BAUD ((F_CPU+UART_BAUD*8)/(UART_BAUD*16)-1)   // clever runden, i = (a+b/2)/b


/**
* Berechnet die reale Bausrate zur Berechnung des Fehlers.
*/
#define UART_BAUD_REAL (F_CPU/(16*(UART_UBRR_BAUD+1)))     		// Reale Baudrate


/**
* Berechnet den Fehler bei der Baudrateneinstellung
*/
#define UART_BAUD_ERROR ((UART_BAUD_REAL*1000)/UART_BAUD) 		// Fehler in Promille, 1000 = kein Fehler.

#if ((UART_BAUD_ERROR<990) || (UART_BAUD_ERROR>1010))
  #error Systematischer Fehler der Baudrate grsser 1% und damit zu hoch! 
#endif


/**
* UART_ISR Gibt an, ob ein Interrupt beim Empfangen von Zeichen ausgelst werden soll (Globales Interrupt Flag wird nicht gesetzt).
* <br>UART_ISR=0 --> Rx Interrupt deaktiviert
* <br>UART_ISR=1 --> Rx Interrupt aktiviert
*/
#define UART_ISR 1

/**
* gibt an, ob mit einem Timeout gearbeitet werden soll (=1 ja, =0 nein)
*/
#define UART_TIMEOUT_ON 1    // mit uart timeout arbieten = 1, ohne uart timeout arbeiten = 0	

/**
* gibt an, ob mit bei dem Timeout mit einer Timer Interruptroutine gearbeitet werden soll (=1 ja, =0 nein)
*/

#define UART_TIMEOUT_ISR 1
/**
* dient als Vergleichswert zur Variablen \ref UART_TIMEOUT_BOUND fr das Arbeiten mit einem Timeout
* <br> Dies sollte abhngig von der jeweiligen Timer-Interruptroutine eingestellt werden und so bemessen sein, dass ca 200ms bis zum auslsen eines Timeout verstreichen.
* <br> (Zeit=100ms=\ref UART_TIMEOUT_BOUND*'Zeit bis zum auslsen der Timer-Interruptroutine'; Bsp. ist Timer-Interruptroutine auf 2ms eingestellt, so sollte 'uart_timeout_bound' 100 sein).
* <br> Ist \ref UART_TIMEOUT_ISR = 0, so wird nicht mit einer Interuptroutine gearbeitet und \ref UART_TIMEOUT_BOUND gibt direkt die Zeit in Millisekunden an (zwischen 1..254).
*/
#define UART_TIMEOUT_BOUND 100 

/**
* Definiert die maximale Anzahl von Zeichen, welche ber die Rx-Interruptroutine empfangen werden knnen.
* <br> Dient damit auch zum Festlegen der Gre der globalen Variable /ref uart_string.
*/
#define UART_MAXSTRLEN 380

//Portierung der Register- und Bitnamen auf ATmega168A
#ifdef __AVR_ATmega168A__
	#define USART_RXC_vect USART_RX_vect
	#define TIMER2_COMP_vect TIMER2_COMPA_vect
	#define UCSRA UCSR0A
	#define UCSRB UCSR0B
	#define UCSRC UCSR0C
	#define UDRE UDRE0
	#define UDR UDR0
	#define RXEN RXEN0
	#define TXEN TXEN0
	#define RXCIE RXCIE0
	#define RXC RXC0
	#define UBRRH UBRR0H
	#define UBRRL UBRR0L
	#define UCSZ0 UCSZ00
	#define UCSZ1 UCSZ01
	
#endif


/**
* globale Variable zum Zhlen der Timer-Interrupts
*/
extern volatile uint8_t uart_timeout;

/**
* Definiert globale Zeichenkette, welche empfangene Zeichen der Rx-Interruptroutine beinhaltet.
* <br> Die Gre der Zeichenkette wird durch maxsrtlen+1 festgelegt (maxstrlen+1 --> maxstrlen Zeichen + 'Stringende-Zeichen').
*/
extern volatile char uart_string[UART_MAXSTRLEN+1];
/**
* Das Flag signalisiert der Hauptanwendung, dass eine komplette Zeichenkette in der Rx-Interruptroutine empfangen wurde und verarbeitet werden kann.
* <br> Eine komplette Zeichenkette wird durch das Erreichen von 'maxstrlen' oder dem empfangenen Zeichen fr 'lf' signalisiert.
* <br> Um neue Zeichenketten zu empfangen, muss das Flag wieder rckgesetzt werden (=0).
* <br> str_complete=1 --> Zeichenkette komplett
* <br> str_complete=0 --> Zeichenkette nicht komplett (warten bis maxstrlen-Zeichen oder 'lf')
*/
extern volatile uint8_t uart_str_complete;
/**
* zhlt die Anzahl der empfangenen Zeichen.
*/
extern volatile uint8_t uart_str_count;

extern void uart_init(void);
//uint8_t uart_receive(void);
extern int uart_receive(FILE *dummy);
extern uint8_t uart_gets(char* buffer, uint8_t maxlen);
//uint8_t uart_sendc(unsigned char c);
extern int uart_sendc(char c, FILE *dummy);
extern void uart_sends(char *s);
extern void uart_clear(void);
